@@ -16,13 +16,15 @@ DEFAULT_PORT_NUMBER = 9192
 
 class ShoppingListItem(object):
     def __init__(self, parsed_json):
-        self._id = parse_int(parsed_json.get('id'))
-        self._product_id = parse_int(parsed_json.get('product_id', None))
-        self._note = parsed_json.get('note', None)
-        self._amount = parse_float(parsed_json.get('amount'), 0)
-        self._row_created_timestamp = parse_date(parsed_json.get('row_created_timestamp', None))
-        self._shopping_list_id = parse_int(parsed_json.get('shopping_list_id'))
-        self._done = parse_int(parsed_json.get('done'))
+        self._id = parse_int(parsed_json.get("id"))
+        self._product_id = parse_int(parsed_json.get("product_id", None))
+        self._note = parsed_json.get("note", None)
+        self._amount = parse_float(parsed_json.get("amount"), 0)
+        self._row_created_timestamp = parse_date(
+            parsed_json.get("row_created_timestamp", None)
+        )
+        self._shopping_list_id = parse_int(parsed_json.get("shopping_list_id"))
+        self._done = parse_int(parsed_json.get("done"))
 
     @property
     def id(self) -> int:
@@ -40,19 +42,22 @@ class ShoppingListItem(object):
     def amount(self) -> float:
         return self._amount
 
+
 class MealPlanResponse(object):
     def __init__(self, parsed_json):
-        self._id = parse_int(parsed_json.get('id'))
-        self._day = parse_date(parsed_json.get('day'))
-        self._type = parsed_json.get('type')
-        self._recipe_id = parse_int(parsed_json.get('recipe_id'))
-        self._recipe_servings = parse_int(parsed_json.get('recipe_servings'))
-        self._note = parsed_json.get('note', None)
-        self._product_id = parsed_json.get('product_id')
-        self._product_amount = parse_float(parsed_json.get('product_amount'), 0)
-        self._product_qu_id = parsed_json.get('product_qu_id')
-        self._row_created_timestamp = parse_date(parsed_json.get('row_created_timestamp'))
-        self._userfields = parsed_json.get('userfields')
+        self._id = parse_int(parsed_json.get("id"))
+        self._day = parse_date(parsed_json.get("day"))
+        self._type = parsed_json.get("type")
+        self._recipe_id = parse_int(parsed_json.get("recipe_id"))
+        self._recipe_servings = parse_int(parsed_json.get("recipe_servings"))
+        self._note = parsed_json.get("note", None)
+        self._product_id = parsed_json.get("product_id")
+        self._product_amount = parse_float(parsed_json.get("product_amount"), 0)
+        self._product_qu_id = parsed_json.get("product_qu_id")
+        self._row_created_timestamp = parse_date(
+            parsed_json.get("row_created_timestamp")
+        )
+        self._userfields = parsed_json.get("userfields")
 
     @property
     def id(self) -> int:
@@ -77,14 +82,16 @@ class MealPlanResponse(object):
 
 class RecipeDetailsResponse(object):
     def __init__(self, parsed_json):
-        self._id = parse_int(parsed_json.get('id'))
-        self._name = parsed_json.get('name')
-        self._description = parsed_json.get('description')
-        self._base_servings = parse_int(parsed_json.get('base_servings'))
-        self._desired_servings = parse_int(parsed_json.get('desired_servings'))
-        self._picture_file_name = parsed_json.get('picture_file_name')
-        self._row_created_timestamp = parse_date(parsed_json.get('row_created_timestamp'))
-        self._userfields = parsed_json.get('userfields')
+        self._id = parse_int(parsed_json.get("id"))
+        self._name = parsed_json.get("name")
+        self._description = parsed_json.get("description")
+        self._base_servings = parse_int(parsed_json.get("base_servings"))
+        self._desired_servings = parse_int(parsed_json.get("desired_servings"))
+        self._picture_file_name = parsed_json.get("picture_file_name")
+        self._row_created_timestamp = parse_date(
+            parsed_json.get("row_created_timestamp")
+        )
+        self._userfields = parsed_json.get("userfields")
 
     @property
     def id(self) -> int:
@@ -110,21 +117,26 @@ class RecipeDetailsResponse(object):
     def picture_file_name(self) -> str:
         return self._picture_file_name
 
+
 class QuantityUnitData(object):
     def __init__(self, parsed_json):
-        self._id = parse_int(parsed_json.get('id'))
-        self._name = parsed_json.get('name')
-        self._name_plural = parsed_json.get('name_plural')
-        self._description = parsed_json.get('description')
-        self._row_created_timestamp = parse_date(parsed_json.get('row_created_timestamp'))
+        self._id = parse_int(parsed_json.get("id"))
+        self._name = parsed_json.get("name")
+        self._name_plural = parsed_json.get("name_plural")
+        self._description = parsed_json.get("description")
+        self._row_created_timestamp = parse_date(
+            parsed_json.get("row_created_timestamp")
+        )
 
 
 class LocationData(object):
     def __init__(self, parsed_json):
-        self._id = parse_int(parsed_json.get('id'))
-        self._name = parsed_json.get('name')
-        self._description = parsed_json.get('description')
-        self._row_created_timestamp = parse_date(parsed_json.get('row_created_timestamp'))
+        self._id = parse_int(parsed_json.get("id"))
+        self._name = parsed_json.get("name")
+        self._description = parsed_json.get("description")
+        self._row_created_timestamp = parse_date(
+            parsed_json.get("row_created_timestamp")
+        )
 
     @property
     def id(self) -> int:
@@ -141,21 +153,29 @@ class LocationData(object):
 
 class ProductData(object):
     def __init__(self, parsed_json):
-        self._id = parse_int(parsed_json.get('id'))
-        self._name = parsed_json.get('name')
-        self._description = parsed_json.get('description', None)
-        self._location_id = parse_int(parsed_json.get('location_id', None))
-        self._product_group_id = parse_int(parsed_json.get('product_group_id', None))
-        self._qu_id_stock = parse_int(parsed_json.get('qu_id_stock', None))
-        self._qu_id_purchase = parse_int(parsed_json.get('qu_id_purchsase', None))
-        self._qu_factor_purchase_to_stock = parse_float(parsed_json.get('qu_factor_purchase_to_stock', None))
-        self._picture_file_name = parsed_json.get('picture_file_name', None)
-        self._allow_partial_units_in_stock = bool(parsed_json.get('allow_partial_units_in_stock', None) == "true")
-        self._row_created_timestamp = parse_date(parsed_json.get('row_created_timestamp', None))
-        self._min_stock_amount = parse_int(parsed_json.get('min_stock_amount', None), 0)
-        self._default_best_before_days = parse_int(parsed_json.get('default_best_before_days', None))
+        self._id = parse_int(parsed_json.get("id"))
+        self._name = parsed_json.get("name")
+        self._description = parsed_json.get("description", None)
+        self._location_id = parse_int(parsed_json.get("location_id", None))
+        self._product_group_id = parse_int(parsed_json.get("product_group_id", None))
+        self._qu_id_stock = parse_int(parsed_json.get("qu_id_stock", None))
+        self._qu_id_purchase = parse_int(parsed_json.get("qu_id_purchsase", None))
+        self._qu_factor_purchase_to_stock = parse_float(
+            parsed_json.get("qu_factor_purchase_to_stock", None)
+        )
+        self._picture_file_name = parsed_json.get("picture_file_name", None)
+        self._allow_partial_units_in_stock = bool(
+            parsed_json.get("allow_partial_units_in_stock", None) == "true"
+        )
+        self._row_created_timestamp = parse_date(
+            parsed_json.get("row_created_timestamp", None)
+        )
+        self._min_stock_amount = parse_int(parsed_json.get("min_stock_amount", None), 0)
+        self._default_best_before_days = parse_int(
+            parsed_json.get("default_best_before_days", None)
+        )
 
-        barcodes_raw = parsed_json.get('barcode', "")
+        barcodes_raw = parsed_json.get("barcode", "")
         if barcodes_raw is None:
             self._barcodes = None
         else:
@@ -180,28 +200,30 @@ class ProductData(object):
 
 class ChoreData(object):
     def __init__(self, parsed_json):
-        self.id = parse_int(parsed_json.get('id'))
-        self.name = parsed_json.get('name')
-        self.description = parsed_json.get('description')
-        self.period_type = parsed_json.get('period_type')
-        self.period_config = parsed_json.get('period_config')
-        self.period_days = parse_int(parsed_json.get('period_days'))
-        self.track_date_only = parsed_json.get('track_date_only')
-        self.rollover = parsed_json.get('rollover')
-        self.assignment_type = parsed_json.get('assignment_type')
-        self.assignment_config = parsed_json.get('assignment_config')
-        self.next_execution_assigned_to_user_id = parse_int('next_execution_assigned_to_user_id')
-        self.userfields = parsed_json.get('userfields')
+        self.id = parse_int(parsed_json.get("id"))
+        self.name = parsed_json.get("name")
+        self.description = parsed_json.get("description")
+        self.period_type = parsed_json.get("period_type")
+        self.period_config = parsed_json.get("period_config")
+        self.period_days = parse_int(parsed_json.get("period_days"))
+        self.track_date_only = parsed_json.get("track_date_only")
+        self.rollover = parsed_json.get("rollover")
+        self.assignment_type = parsed_json.get("assignment_type")
+        self.assignment_config = parsed_json.get("assignment_config")
+        self.next_execution_assigned_to_user_id = parse_int(
+            "next_execution_assigned_to_user_id"
+        )
+        self.userfields = parsed_json.get("userfields")
 
 
 class UserDto(object):
     def __init__(self, parsed_json):
-        self._id = parse_int(parsed_json.get('id'))
+        self._id = parse_int(parsed_json.get("id"))
 
-        self._username = parsed_json.get('username')
-        self._first_name = parsed_json.get('first_name')
-        self._last_name = parsed_json.get('last_name')
-        self._display_name = parsed_json.get('display_name')
+        self._username = parsed_json.get("username")
+        self._first_name = parsed_json.get("first_name")
+        self._last_name = parsed_json.get("last_name")
+        self._display_name = parsed_json.get("display_name")
 
     @property
     def id(self) -> int:
@@ -226,9 +248,11 @@ class UserDto(object):
 
 class CurrentChoreResponse(object):
     def __init__(self, parsed_json):
-        self._chore_id = parse_int(parsed_json.get('chore_id'), None)
-        self._last_tracked_time = parse_date(parsed_json.get('last_tracked_time'))
-        self._next_estimated_execution_time = parse_date(parsed_json.get('next_estimated_execution_time'))
+        self._chore_id = parse_int(parsed_json.get("chore_id"), None)
+        self._last_tracked_time = parse_date(parsed_json.get("last_tracked_time"))
+        self._next_estimated_execution_time = parse_date(
+            parsed_json.get("next_estimated_execution_time")
+        )
 
     @property
     def chore_id(self) -> int:
@@ -245,11 +269,11 @@ class CurrentChoreResponse(object):
 
 class CurrentStockResponse(object):
     def __init__(self, parsed_json):
-        self._product_id = parse_int(parsed_json.get('product_id'))
-        self._amount = parse_float(parsed_json.get('amount'))
-        self._best_before_date = parse_date(parsed_json.get('best_before_date'))
-        self._amount_opened = parse_float(parsed_json.get('amount_opened'))
-        self._product = ProductData(parsed_json.get('product'))
+        self._product_id = parse_int(parsed_json.get("product_id"))
+        self._amount = parse_float(parsed_json.get("amount"))
+        self._best_before_date = parse_date(parsed_json.get("best_before_date"))
+        self._amount_opened = parse_float(parsed_json.get("amount_opened"))
+        self._product = ProductData(parsed_json.get("product"))
 
     @property
     def product_id(self) -> int:
@@ -274,10 +298,12 @@ class CurrentStockResponse(object):
 
 class MissingProductResponse(object):
     def __init__(self, parsed_json):
-        self._product_id = parse_int(parsed_json.get('id'))
-        self._name = parsed_json.get('name')
-        self._amount_missing = parse_float(parsed_json.get('amount_missing'))
-        self._is_partly_in_stock = bool(parse_int(parsed_json.get('is_partly_in_stock')))
+        self._product_id = parse_int(parsed_json.get("id"))
+        self._name = parsed_json.get("name")
+        self._amount_missing = parse_float(parsed_json.get("amount_missing"))
+        self._is_partly_in_stock = bool(
+            parse_int(parsed_json.get("is_partly_in_stock"))
+        )
 
     @property
     def product_id(self) -> int:
@@ -298,9 +324,18 @@ class MissingProductResponse(object):
 
 class CurrentVolatilStockResponse(object):
     def __init__(self, parsed_json):
-        self._expiring_products = [CurrentStockResponse(product) for product in parsed_json.get('expiring_products')]
-        self._expired_products = [CurrentStockResponse(product) for product in parsed_json.get('expired_products')]
-        self._missing_products = [MissingProductResponse(product) for product in parsed_json.get('missing_products')]
+        self._expiring_products = [
+            CurrentStockResponse(product)
+            for product in parsed_json.get("expiring_products")
+        ]
+        self._expired_products = [
+            CurrentStockResponse(product)
+            for product in parsed_json.get("expired_products")
+        ]
+        self._missing_products = [
+            MissingProductResponse(product)
+            for product in parsed_json.get("missing_products")
+        ]
 
     @property
     def expiring_products(self) -> List[CurrentStockResponse]:
@@ -317,19 +352,25 @@ class CurrentVolatilStockResponse(object):
 
 class ProductDetailsResponse(object):
     def __init__(self, parsed_json):
-        self._last_purchased = parse_date(parsed_json.get('last_purchased'))
-        self._last_used = parse_date(parsed_json.get('last_used'))
-        self._stock_amount = parse_int(parsed_json.get('stock_amount'))
-        self._stock_amount_opened = parse_int(parsed_json.get('stock_amount_opened'))
-        self._next_best_before_date = parse_date(parsed_json.get('next_best_before_date'))
-        self._last_price = parse_float(parsed_json.get('last_price'))
+        self._last_purchased = parse_date(parsed_json.get("last_purchased"))
+        self._last_used = parse_date(parsed_json.get("last_used"))
+        self._stock_amount = parse_int(parsed_json.get("stock_amount"))
+        self._stock_amount_opened = parse_int(parsed_json.get("stock_amount_opened"))
+        self._next_best_before_date = parse_date(
+            parsed_json.get("next_best_before_date")
+        )
+        self._last_price = parse_float(parsed_json.get("last_price"))
 
-        self._product = ProductData(parsed_json.get('product'))
+        self._product = ProductData(parsed_json.get("product"))
 
-        self._quantity_unit_purchase = QuantityUnitData(parsed_json.get('quantity_unit_purchase'))
-        self._quantity_unit_stock = QuantityUnitData(parsed_json.get('quantity_unit_stock'))
+        self._quantity_unit_purchase = QuantityUnitData(
+            parsed_json.get("quantity_unit_purchase")
+        )
+        self._quantity_unit_stock = QuantityUnitData(
+            parsed_json.get("quantity_unit_stock")
+        )
 
-        self._location = LocationData(parsed_json.get('location'))
+        self._location = LocationData(parsed_json.get("location"))
 
     @property
     def last_purchased(self) -> datetime:
@@ -362,12 +403,14 @@ class ProductDetailsResponse(object):
 
 class ChoreDetailsResponse(object):
     def __init__(self, parsed_json):
-        self._chore = ChoreData(parsed_json.get('chore'))
-        self._last_tracked = parse_date(parsed_json.get('last_tracked'))
-        self._next_estimated_execution_time = parse_date(parsed_json.get('next_estimated_execution_time'))
-        self._track_count = parse_int(parsed_json.get('track_count'))
+        self._chore = ChoreData(parsed_json.get("chore"))
+        self._last_tracked = parse_date(parsed_json.get("last_tracked"))
+        self._next_estimated_execution_time = parse_date(
+            parsed_json.get("next_estimated_execution_time")
+        )
+        self._track_count = parse_int(parsed_json.get("track_count"))
 
-        next_user = parsed_json.get('next_execution_assigned_user')
+        next_user = parsed_json.get("next_execution_assigned_user")
         if next_user is not None:
             self._next_execution_assigned_user = UserDto(next_user)
         else:
@@ -376,7 +419,7 @@ class ChoreDetailsResponse(object):
         if self._last_tracked is None:
             self._last_done_by = None
         else:
-            self._last_done_by = UserDto(parsed_json.get('last_done_by'))
+            self._last_done_by = UserDto(parsed_json.get("last_done_by"))
 
     @property
     def chore(self) -> ChoreData:
@@ -412,34 +455,32 @@ class TransactionType(Enum):
 
 class TaskResponse(object):
     def __init__(self, parsed_json):
-        self.id = parse_int(parsed_json.get('id'))
-        self.name = parsed_json.get('name')
-        self.description = parsed_json.get('description')
-        self.due_date = parse_date(parsed_json.get('due_date'))
-        self.done = parse_int(parsed_json.get('done'))
-        self.done_timestamp = parse_date(parsed_json.get('done_timestamp'))
-        self.category_id = parse_int(parsed_json.get('category_id'))
-        self.assigned_to_user_id = parse_int(parsed_json.get('assigned_to_user_id'))
-        self.userfields = parsed_json.get('userfields')
+        self.id = parse_int(parsed_json.get("id"))
+        self.name = parsed_json.get("name")
+        self.description = parsed_json.get("description")
+        self.due_date = parse_date(parsed_json.get("due_date"))
+        self.done = parse_int(parsed_json.get("done"))
+        self.done_timestamp = parse_date(parsed_json.get("done_timestamp"))
+        self.category_id = parse_int(parsed_json.get("category_id"))
+        self.assigned_to_user_id = parse_int(parsed_json.get("assigned_to_user_id"))
+        self.userfields = parsed_json.get("userfields")
 
 
 class GrocyApiClient(object):
-    def __init__(self, base_url, api_key, port: int = DEFAULT_PORT_NUMBER, verify_ssl=True):
-        self._base_url = '{}:{}/api/'.format(base_url, port)
+    def __init__(
+        self, base_url, api_key, port: int = DEFAULT_PORT_NUMBER, verify_ssl=True
+    ):
+        self._base_url = "{}:{}/api/".format(base_url, port)
         self._api_key = api_key
         self._verify_ssl = verify_ssl
         if self._api_key == "demo_mode":
             self._headers = {"accept": "application/json"}
         else:
-            self._headers = {
-                "accept": "application/json",
-                "GROCY-API-KEY": api_key
-            }
+            self._headers = {"accept": "application/json", "GROCY-API-KEY": api_key}
 
     def _do_get_request(self, end_url: str):
         req_url = urljoin(self._base_url, end_url)
-        resp = requests.get(
-            req_url, verify=self._verify_ssl, headers=self._headers)
+        resp = requests.get(req_url, verify=self._verify_ssl, headers=self._headers)
         resp.raise_for_status()
         if len(resp.content) > 0:
             return resp.json()
@@ -447,9 +488,8 @@ class GrocyApiClient(object):
     def _do_post_request(self, end_url: str, data: dict):
         req_url = urljoin(self._base_url, end_url)
         resp = requests.post(
-            req_url, verify=self._verify_ssl,
-            headers=self._headers,
-            data=data)
+            req_url, verify=self._verify_ssl, headers=self._headers, data=data
+        )
         resp.raise_for_status()
         if len(resp.content) > 0:
             return resp.json()
@@ -457,16 +497,15 @@ class GrocyApiClient(object):
     def _do_put_request(self, end_url: str, data):
         req_url = urljoin(self._base_url, end_url)
         up_header = self._headers.copy()
-        up_header['accept'] = '*/*'
+        up_header["accept"] = "*/*"
         if isinstance(data, dict):
-            up_header['Content-Type'] = 'application/json'
+            up_header["Content-Type"] = "application/json"
             data = json.dumps(data)
         else:
-            up_header['Content-Type'] = 'application/octet-stream'
+            up_header["Content-Type"] = "application/octet-stream"
         resp = requests.put(
-            req_url, verify=self._verify_ssl,
-            headers=up_header,
-            data=data)
+            req_url, verify=self._verify_ssl, headers=up_header, data=data
+        )
         resp.raise_for_status()
         if len(resp.content) > 0:
             return resp.json()
@@ -495,40 +534,54 @@ class GrocyApiClient(object):
         if parsed_json:
             return ChoreDetailsResponse(parsed_json)
 
-    def execute_chore(self, chore_id: int, done_by: int = None, tracked_time: datetime = datetime.now()):
+    def execute_chore(
+        self,
+        chore_id: int,
+        done_by: int = None,
+        tracked_time: datetime = datetime.now(),
+    ):
         # Grocy API expects UTC time; time returned from datetime.now() is local time without timezone
         # information, so timezone information must be attached.
         local_tz = get_localzone()
         localized_tracked_time = local_tz.localize(tracked_time)
 
-        data = {
-            "tracked_time": localized_tracked_time.isoformat()
-        }
+        data = {"tracked_time": localized_tracked_time.isoformat()}
 
         if done_by is not None:
             data["done_by"] = done_by
 
         return self._do_post_request(f"chores/{chore_id}/execute", data)
 
-    def add_product(self, product_id, amount: float, price: float, best_before_date: datetime = None,
-                    transaction_type: TransactionType = TransactionType.PURCHASE):
+    def add_product(
+        self,
+        product_id,
+        amount: float,
+        price: float,
+        best_before_date: datetime = None,
+        transaction_type: TransactionType = TransactionType.PURCHASE,
+    ):
         data = {
             "amount": amount,
             "transaction_type": transaction_type.value,
-            "price": price
+            "price": price,
         }
 
         if best_before_date is not None:
-            data["best_before_date"] = best_before_date.strftime('%Y-%m-%d')
+            data["best_before_date"] = best_before_date.strftime("%Y-%m-%d")
 
         return self._do_post_request(f"stock/products/{product_id}/add", data)
 
-    def consume_product(self, product_id: int, amount: float = 1, spoiled: bool = False,
-                        transaction_type: TransactionType = TransactionType.CONSUME):
+    def consume_product(
+        self,
+        product_id: int,
+        amount: float = 1,
+        spoiled: bool = False,
+        transaction_type: TransactionType = TransactionType.CONSUME,
+    ):
         data = {
             "amount": amount,
             "spoiled": spoiled,
-            "transaction_type": transaction_type.value
+            "transaction_type": transaction_type.value,
         }
 
         self._do_post_request(f"stock/products/{product_id}/consume", data)
@@ -540,32 +593,32 @@ class GrocyApiClient(object):
     def add_missing_product_to_shopping_list(self, shopping_list_id: int = None):
         data = None
         if shopping_list_id:
-            data = {
-                "list_id": shopping_list_id
-            }
+            data = {"list_id": shopping_list_id}
 
         self._do_post_request("stock/shoppinglist/add-missing-products", data)
 
-    def add_product_to_shopping_list(self, product_id: int, shopping_list_id: int = 1, amount: int = 1):
+    def add_product_to_shopping_list(
+        self, product_id: int, shopping_list_id: int = 1, amount: int = 1
+    ):
         data = {
             "product_id": product_id,
             "list_id": shopping_list_id,
-            "product_amount": amount
+            "product_amount": amount,
         }
         self._do_post_request("stock/shoppinglist/add-product", data)
 
     def clear_shopping_list(self, shopping_list_id: int = 1):
-        data = {
-            "list_id": shopping_list_id
-        }
+        data = {"list_id": shopping_list_id}
 
         self._do_post_request("stock/shoppinglist/clear", data)
 
-    def remove_product_in_shopping_list(self, product_id: int, shopping_list_id: int = 1, amount: int = 1):
+    def remove_product_in_shopping_list(
+        self, product_id: int, shopping_list_id: int = 1, amount: int = 1
+    ):
         data = {
             "product_id": product_id,
             "list_id": shopping_list_id,
-            "product_amount": amount
+            "product_amount": amount,
         }
         self._do_post_request("stock/shoppinglist/remove-product", data)
 
@@ -574,9 +627,9 @@ class GrocyApiClient(object):
         return [LocationData(response) for response in parsed_json]
 
     def upload_product_picture(self, product_id: int, pic_path: str):
-        b64fn = base64.b64encode('{}.jpg'.format(product_id).encode('ascii'))
+        b64fn = base64.b64encode("{}.jpg".format(product_id).encode("ascii"))
         req_url = "files/productpictures/" + str(b64fn, "utf-8")
-        with open(pic_path, 'rb') as pic:
+        with open(pic_path, "rb") as pic:
             self._do_put_request(req_url, pic)
 
     def update_product_pic(self, product_id: int):
@@ -589,23 +642,27 @@ class GrocyApiClient(object):
         return self._do_get_request(url)
 
     def set_userfields(self, entity: str, object_id: int, key: str, value):
-        data = {
-            key: value
-        }
+        data = {key: value}
         self._do_put_request(f"userfields/{entity}/{object_id}", data)
 
     def get_last_db_changed(self):
         resp = self._do_get_request("system/db-changed-time")
-        last_change_timestamp = parse_date(resp.get('changed_time'))
+        last_change_timestamp = parse_date(resp.get("changed_time"))
         return last_change_timestamp
 
     def get_tasks(self) -> List[TaskResponse]:
         parsed_json = self._do_get_request("tasks")
         return [TaskResponse(data) for data in parsed_json]
 
-    def complete_task(self, task_id: int):
+    def complete_task(self, task_id: int, done_time: datetime = datetime.now()):
         url = f"tasks/{task_id}/complete"
-        self._do_post_request(url)
+        # Grocy API expects UTC time; time returned from datetime.now() is local time without timezone
+        # information, so timezone information must be attached.
+        local_tz = get_localzone()
+        localized_done_time = local_tz.localize(done_time)
+
+        data = {"done_time": localized_done_time.isoformat()}
+        self._do_post_request(url, data)
 
     def get_meal_plan(self) -> List[MealPlanResponse]:
         parsed_json = self._do_get_request("objects/meal_plan")
@@ -615,3 +672,6 @@ class GrocyApiClient(object):
         parsed_json = self._do_get_request(f"objects/recipes/{object_id}")
         if parsed_json:
             return RecipeDetailsResponse(parsed_json)
+
+    def add_generic(self, entity_type: str, data: object):
+        self._do_post_request(f"objects/{entity_type}", data)
